/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 *
 */

/**
 * 
 */
package se.vgregion.alfrescoclient.service;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.codec.binary.Base64;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.MappingJacksonHttpMessageConverter;
import org.springframework.web.client.RestTemplate;

import se.vgregion.alfrescoclient.domain.Site;

/**
 * This service is responsible for the communication with Alfresco server using it´s REST API.
 * 
 * @author Simon Göransson
 * @author Björn Ryding
 * 
 */
public class AlfrescoService {

    private static final String API_PEOPLE = "/alfresco/wcservice/api/people/";
    private static final String SITES = "/sites";
    private static final String DASHBOARD = "/dashboard";
    private static final String PAGE_SITES = "/share/page/site/";
    private static final String PUBLIC_SITES = "/alfresco/wcservice/vgr/public-sites";

    private final String server;
    private final String SSOUserHeaderField;

    /**
     * The constructor for the service.
     * 
     * @param server
     *            - Alfresco server base URL.
     */
    public AlfrescoService(String server, String SSOUserHeaderField) {
        this.server = server;
        this.SSOUserHeaderField = SSOUserHeaderField;
    }

    /**
     * Initialises the RestTemplate.
     * 
     * @return the RestTemplate
     */
    private RestTemplate initJsonRestTemplate() {
        RestTemplate template = new RestTemplate();
        HttpMessageConverter<?>[] httpMessageConverters =
                new HttpMessageConverter<?>[]{new MappingJacksonHttpMessageConverter()};
        template.setMessageConverters(Arrays.asList(httpMessageConverters));

        return template;
    }

    /**
     * Sets up the Http request entity.
     * 
     * @param ssoUser
     *            user id for the user that the API calls should performed as.
     * @return Http entity
     */
    private HttpEntity<String> setUpHttpEntity(String ssoUser) {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.add(SSOUserHeaderField, ssoUser);
        HttpEntity<String> httpEntity = new HttpEntity<String>(httpHeaders);

        return httpEntity;

    }

    /**
     * Adds urls to the Alfresco share dashboard for each site.
     * 
     * @param list
     * @return sites with dashboard URLs
     */
    private List<Site> addAlfrescoShareDashboardURLs(List<Site> sites, String csIframPage,
            String portletInstance) {
        for (Site site : sites) {
            String shareLink = server + PAGE_SITES + site.getShortName() + DASHBOARD;

            String encodeUrl = Base64.encodeBase64URLSafeString(shareLink.getBytes());

            site.setShareUrl(csIframPage + "/-/autologin/" + portletInstance + "/" + encodeUrl);

        }
        return sites;
    }

    /**
     * This method returns the Alfresco sites that a user is a member of.
     * 
     * @param user
     *            - user id
     * @return Alfresco sites for a user.
     */
    public List<Site> getSitesByUser(String user, String csIframPage, String portletInstance) {
        RestTemplate template = initJsonRestTemplate();
        String apiURL = server + API_PEOPLE + user + SITES;
        HttpEntity<String> httpEntity = setUpHttpEntity(user);

        ResponseEntity<Site[]> httpResponseEntity =
                template.exchange(apiURL, HttpMethod.GET, httpEntity, Site[].class,
                        new HashMap<String, String>());

        List<Site> sites = Arrays.asList(httpResponseEntity.getBody());
        List<Site> sitesWithDashboardURLs =
                addAlfrescoShareDashboardURLs(sites, csIframPage, portletInstance);

        return sitesWithDashboardURLs;
    }

    /**
     * This method returns the Alfresco sites that a user is a member of.
     * 
     * @param user
     *            - user id
     * @return Alfresco sites for a user.
     */
    public List<Site> getPublicSites(String user, String csIframPage, String portletInstance) {
        RestTemplate template = initJsonRestTemplate();
        String apiURL = server + PUBLIC_SITES;
        HttpEntity<String> httpEntity = setUpHttpEntity(user);

        ResponseEntity<Site[]> httpResponseEntity =
                template.exchange(apiURL, HttpMethod.GET, httpEntity, Site[].class,
                        new HashMap<String, String>());

        List<Site> sites = Arrays.asList(httpResponseEntity.getBody());
        List<Site> sitesWithDashboardURLs =
                addAlfrescoShareDashboardURLs(sites, csIframPage, portletInstance);

        return sitesWithDashboardURLs;
    }
}
